#include "errors.h"
#include "parsing.h"
#include "datagram.h"
#include "machines.h"
#include "system.h"

#include <UnitTest++/UnitTest++.h>
#include <array>
#include <memory>
#include <iostream>
#include <stdexcept>

using namespace std;

// A system of four connected machines. They are used in the following unit
// tests
class SystemF
{
public:
    array<shared_ptr<Node>, 4> ms_;

    SystemF()
    {
        ms_[0] = make_shared<Node>("L", IP_address("192.168.0.1"));
        ms_[1] = make_shared<Node>("S", IP_address("140.112.239.8"));
        ms_[2] = make_shared<Node>("T", IP_address("140.112.253.11"));
        ms_[3] = make_shared<Node>("W", IP_address("8.8.8.8"));
        for (size_t i = 1; i < 4; ++i) {
            ms_[0]->connect(ms_[i]);
            ms_[i]->connect(ms_[0]);
        }
    }

    ~SystemF()
    {
        // Break the connection between the machines to free the memory
        for (size_t i = 1; i < 4; ++i) {
            ms_[0]->disconnect(ms_[i]);
            ms_[i]->disconnect(ms_[0]);
        }
    }
};

// Test the specification "If the destination of the Datagram is
// in the connected machines, send the datagram to that machine."
TEST(SEND_BEST_MACHINE_DESTINATION_MATCH)
{
    SystemF sys;
    sys.ms_[0]->allocate_datagram(sys.ms_[2]->get_ip(), "ping");
    CHECK_EQUAL(1, sys.ms_[0]->send()); // successfully sent one datagram
    CHECK_EQUAL("ping", sys.ms_[2]->release_datagram());
}

// Test the specification "Otherwise, send it to the connected machine
// whose first number of IP address is closest to the first number of
// the destination."
TEST(SEND_BEST_MACHINE_CLOSEST_FIRST_NUMBER)
{
    SystemF sys;
    sys.ms_[0]->allocate_datagram(IP_address("9.8.7.6"), "pong");
    CHECK_EQUAL(1, sys.ms_[0]->send()); // successfully sent one datagram
    CHECK_EQUAL(1, sys.ms_[3]->send()); // ms_[3] is not the destination, but
                                        // we can send it out again
}
